using System;
using System.ComponentModel;
using System.Windows.Forms;

using Smdn.Formats;
using Smdn.Formats.Thbgm;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public class ExtendedTrackListView : TrackListView<ExtendedTrackListViewItem> {
    public event EventHandler TrackEffectPropertyChanged;

    public string Format {
      get { return format; }
      set
      {
        if (format == value)
          return;

        format = value;

        UpdateItems();
      }
    }

    [Browsable(false)]
    [EditorBrowsable(EditorBrowsableState.Never)]
    [Localizable(true)]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    public TrackFormatInfo TrackFormatInfo {
      get { return trackFormatInfo; }
      set
      {
        if (trackFormatInfo == value)
          return;

        trackFormatInfo = value;

        UpdateItems();
      }
    }

    public ExtendedTrackListView()
    {
      Columns.Insert(2, ThbgmExtractor.GetText("OutputFilename"), 200);

      Columns.Add(ThbgmExtractor.GetText("OutputFileLength"), 75);
    }

    public override void UpdateAllTracksItemSubItems()
    {
      base.UpdateAllTracksItemSubItems();

      AllTracksItem.SubItems[1] = new ListViewItem.ListViewSubItem(AllTracksItem, (Product == null) ? string.Empty : Product.GetUnifiedTitle(trackFormatInfo.TitleUnificationOptions));

      AllTracksItem.SubItems.Insert(2, new ListViewItem.ListViewSubItem(AllTracksItem, Texts.GetText("MultipleValues")));

      var totalLength = 0L;

      foreach (var item in TrackItems) {
        if (item.Track != null)
          totalLength += item.Track.TrackInfo.StreamFormat.ToAlignedByteCount(item.Track.Effect.TotalLength);
      }

      AllTracksItem.SubItems.Add(string.Format(SIPrefixFormat.CurrentInfo, "{0:f}", totalLength));
    }

    private void UpdateItems()
    {
      foreach (var item in TrackItems) {
        item.InternalUpdateSubItems();
      }
    }

    internal protected virtual void OnTrackEffectPropertyChanged(EventArgs e)
    {
      var ev = this.TrackEffectPropertyChanged;

      if (ev != null)
        ev(this, e);
    }

    private string format = Config.TrackFormatInfoDefaults.Format;
    private TrackFormatInfo trackFormatInfo = Config.TrackFormatInfoDefaults.TrackFormatInfo.Clone();
  }
}
