using System;
using System.IO;
using System.Xml;

using Smdn.Collections;
using Smdn.Formats.Thbgm;
using Smdn.Xml;

namespace Smdn.Applications.ThbgmExtractor {
  public static class THxxBGM {
    public const string THxxBGMSettingsFileName = "setting.xml";

    public static void ImportTHxxBGMSettings(string file)
    {
      var xml = new XmlDocument();

      xml.Load(file);

      // /THxxBGMSettings/BgmPath/Count
      var count = xml.GetSingleNodeValueOf<int?>("/THxxBGMSettings/BgmPath/Count/text()", NullableConvert.ToInt32IgnoreException) ?? 0;

      for (var n = 1; n <= count; n++) { // 1-based
        // /THxxBGMSettings/BgmPath/Item[n]
        var itemNode = xml.SelectSingleNode(string.Format("/THxxBGMSettings/BgmPath/Item{0}", n));

        if (itemNode == null)
          continue;

        if ((itemNode.GetSingleNodeValueOf<int?>("Use/text()", NullableConvert.ToInt32) ?? 0) != 1)
          // no use, ignore
          continue;

        var guid = itemNode.GetSingleNodeValueOf<Guid?>("Guid/text()", NullableConvert.ToGuidIgnoreException) ?? Guid.Empty;

        var product = ThbgmExtractor.Products.Find(delegate(ProductInfo p) {
          return p.THxxBGMProductGuid == guid;
        });

        if (product == null)
          // unsupported or not found
          return;

        var bgmPath = itemNode.GetSingleNodeValueOf("BgmPath/text()");

        bgmPath = Path.Combine(bgmPath, Path.GetFileName(product.DefaultBgmSourcePath));

        if (ProductInfo.FindMatchedProduct(bgmPath, ThbgmExtractor.Products) != product) // InstalledBgmSourcePath will be updated
          continue;

        Console.WriteLine("imported installed bgm source path: {0} => {1}", product.Title, product.InstalledBgmSourcePath);
      }
    }
  }
}
