// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;

using Smdn.Collections;
using Smdn.Formats.Playlists;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public static class TrackPlaylist {
    public static void Save(string pathPlaylist, IDictionary<Track, string> trackAndPaths, PlaylistFormat format)
    {
      var playlistBaseDirectory = Path.GetDirectoryName(pathPlaylist);
      var product = trackAndPaths.Keys.First().TrackInfo.Product;

      var playlist = new Playlist(trackAndPaths.ConvertAll(delegate(KeyValuePair<Track, string> pair) {
        Uri trackLocation;

        if (Path.GetDirectoryName(pair.Value).StartsWith(playlistBaseDirectory))
          // set relative path
          //trackLocation = new Uri(new Uri(string.Format("file://{0}", playlistBaseDirectory)), "./" + pair.Value.Replace(playlistBaseDirectory, string.Empty));
          trackLocation = new Uri("." + pair.Value.Replace(playlistBaseDirectory, string.Empty), UriKind.Relative);
        else
          // set absolute path
          trackLocation = new Uri(string.Format("file://{0}",pair.Value));

        var entry = new PlaylistEntry(trackLocation);
        var track = pair.Key;

        entry.Title = track.TrackInfo.Title;
        entry.TrackNumber = track.TrackInfo.TrackNumber;
        entry.AlbumTitle = track.TrackInfo.Product.Title;
        entry.Duration = track.Effect.TotalLength;

        return entry;
      }));

      playlist.Creator = product.Creator;
      playlist.CreationDate = DateTimeOffset.Now;
      playlist.Generator = ThbgmExtractor.VersionString;
      playlist.Title = product.Title;

      playlist.Save(pathPlaylist, format);
    }
  }
}
